/***********************************************************************
 
    c++dbc library. A free extensible C++ database connectivity library
    Copyright (C) 2002-2009  Markus Graf, M.Sc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/> 

************************************************************************/
/* 
 * modification history
 *  2002-03-20 gm   reimplementation of database connectivity library as part of expressRM (for more information see http://expressrm.grafs-world.de)
 *  ...        gm   a lot undocumented stuff happened due to several time consuming and occupying tasks as my masters degree a.s.o.
 *  2009-03-21 gm   code-review, readded transaction calls from previous implementation
 */

#if !defined(_IRESULTSET_H)
#define _IRESULTSET_H

#include "types.h"

class IResultSetMetaData;


/**
 * IResultSet holds the results of a query and provides row by row based access to the columns
 */
class CPPDBC_API IResultSet
{
public:
  /**
   * creates a result set
   */
  IResultSet()  {}
  /**
   * destroys a result set
   */
  virtual ~IResultSet() { }
  /**
   * steps to the next row. Must be called before the first call to any data-returning function.
   * @return true if the (next) row exists and its data can be retrieved, false otherwise
   */
  virtual bool next() =0;
  /**
   * returns the number of columns of this resultset
   * @return number of columns
   */
  virtual unsigned int getColumnCount()=0;
  /**
   * retrieves data as string for field \arg field and puts it into \arg buffer if length \arg len of the string is big enough.
   * @param field field-number (begins with index 0)
   * @param buffer pointer to a buffer of characters
   * @param len length of buffer in bytes
   * @param returns the pointer to the buffer string
   */
  virtual char* getString(int field, char* buffer, int len)=0;
  /**
   * returns data as integer value for field \arg field
   * @param field field-number (begins with index 0)
   * @return integer value
   */
  virtual int getInt(int field)=0;
  /**
   * returns data as long value for field \arg field
   * @param field field-number (begins with index 0)
   * @return long value
   */
  virtual long getLong(int field)=0;
  /**
   * returns data as double value for field \arg field
   * @param field field-number (begins with index 0)
   * @return double value
   */
  virtual double getDouble(int field)=0;

  /**
   * returns pointer to the one and only meta information instance about this result set
   * it contains an interface to retrieve information about the columns
   * @return pointer to the one and only meta data for this result set, no new one will be created!
   */
  virtual IResultSetMetaData* getResultSetMetaData()=0;
};

#endif  //_IRESULTSET_H
